#!/usr/bin/env python

import subprocess
import select
import signal
import MySQLdb
import os
import sys
import Rappture
import time
import numpy

def mysql_cli(sqlcmds,parameters):
    cmd = "mysql -h %(dbhost)s -u %(dbuser)s -p%(dbpass)s -e '" + sqlcmds + "'"
    cmd = cmd % parameters
    p = subprocess.Popen(cmd,stdout=subprocess.PIPE,shell=True)
    p.communicate()

installdir = os.path.dirname(__file__)
# solarpv_dump_path = os.path.join(installdir,'solarpv_dump.sql')
solarpv_dump_path = '/data/tools/solarpv/solarpv_2013_08_28.sql'

# set database parameters for connecting and restore
db_params = {
    'backup_file'   : solarpv_dump_path,
    'dbname'        : 'solarPV',
    'dbhost'        : 'localhost',
    'dbuser'        : 'root',
    'dbpass'        : 'ccehub',
}

# initialize the user specified values from gui
user_params = {
    "ActualNumberofHouseholdsMillions"  : 13,
    "SolarPenetration"                  : 0,
    "BatteryPenetration"                : 0,
    "BatteryNamePlate"                  : 2000,
    "SolarNamePlate"                    : 2000,
    "RandomMonthFlag"                   : 1,
    "SpecificMonth"                     : 0,
    "SolarSpecific"                     : 0,
    "BatterywSolar"                     : 1,
    "SpecifiedMonth"                    : 8,
    "SeasonNum"                         : 0,
    "Season"                            : "Summer",
}

# read the user specified values from the gui
driver = Rappture.library(sys.argv[1])

user_params['AppliancesProfile'] = \
    driver.get('input.phase(area).group(profiles).choice(appliances).current')

user_params['MonthlyTempProfile'] = \
    driver.get('input.phase(area).group(profiles).choice(monthly_temp).current')

user_params['SolarIrradiationMeansProfile'] = \
    driver.get('input.phase(area).group(profiles).choice(sol_irr_mean).current')

user_params['SolarIrradiationDevProfile'] = \
    driver.get('input.phase(area).group(profiles).choice(sol_irr_deviation).current')

user_params['CloudCoverProfile'] = \
    driver.get('input.phase(area).group(profiles).choice(cloud_cover).current')

user_params['ActualNumberofHouseholdsMillions'] = \
    float(driver.get('input.phase(area).group(region).number(households).current'))/1.0e6

month = int(driver.get('input.phase(area).group(region).choice(month).current'))

if month in [5,6,7,8,9,10]:
    # specific summer month chosen
    user_params.update({
        'RandomMonthFlag'   : 0,
        'SpecificMonth'     : 1,
        'SpecifiedMonth'    : month,
        'SeasonNum'         : 0,
        'Season'            : "Summer",
    })
elif month in [1,2,3,4,11,12]:
    # specific winter month chosen
    user_params.update({
        'RandomMonthFlag'   : 0,
        'SpecificMonth'     : 1,
        'SpecifiedMonth'    : month,
        'SeasonNum'         : 1,
        'Season'            : "Winter",
    })
elif month == 13:
    # random summer month chosen
    user_params.update({
        'RandomMonthFlag'   : 1,
        'SpecificMonth'     : 0,
        'SpecifiedMonth'    : 8,
        'SeasonNum'         : 0,
        'Season'            : "Summer",
    })
elif month == 14:
    # random winter month chosen
    user_params.update({
        'RandomMonthFlag'   : 1,
        'SpecificMonth'     : 0,
        'SpecifiedMonth'    : 2,
        'SeasonNum'         : 1,
        'Season'            : "Winter",
    })

user_params['SolarPenetration'] = \
    float(driver.get('input.phase(system).number(solar_penetration).current'))

capacity = driver.get('input.phase(system).number(solar_capacity).current')
# user_params['SolarNamePlate'] = Rappture.Units.convert(capacity,"W")
user_params['SolarNamePlate'] = float(capacity)

val = driver.get('input.phase(battery).boolean(with_battery).current')
user_params['BatterywSolar'] = 1 if val == 'yes' else 0

user_params['BatteryPenetration'] = \
    float(driver.get('input.phase(battery).number(battery_penetration).current'))

capacity = driver.get('input.phase(battery).number(battery_capacity).current')
# user_params['BatteryNamePlate'] = Rappture.Units.convert(capacity,"Wh")
user_params['BatteryNamePlate'] = float(capacity)

# what is SolarSpecific, where is it in the gui?
user_params['SolarSpecific'] = 0 if user_params['BatterywSolar'] == 1 else 1


Rappture.Utils.progress(0,"Starting the simulation")

# create and restore the db from the backup file
# populate the database with user values

sqlcmds = "\
    drop database if exists %(dbname)s;\
    create database %(dbname)s;\
    use %(dbname)s;\
    source %(backup_file)s;\
    USE %(dbname)s;\
    CREATE TABLE OtherParameters (\
        ActualNumberofHouseholdsMillions DOUBLE,\
        SolarPenetration DOUBLE,\
        BatteryPenetration DOUBLE,\
        BatteryNamePlate DOUBLE,\
        SolarNamePlate DOUBLE,\
        RandomMonthFlag BOOLEAN,\
        SpecificMonth TINYINT,\
        SolarSpecific TINYINT,\
        BatterywSolar TINYINT,\
        SpecifiedMonth DOUBLE,\
        SeasonNum INT,\
        Season VARCHAR(10)\
    );\
    INSERT INTO OtherParameters (\
        ActualNumberofHouseholdsMillions,\
        SolarPenetration,\
        BatteryPenetration,\
        BatteryNamePlate,\
        SolarNamePlate,\
        RandomMonthFlag,\
        SpecificMonth,\
        SolarSpecific,\
        BatterywSolar,\
        SpecifiedMonth,\
        SeasonNum,\
        Season \
    ) VALUES (\
        %(ActualNumberofHouseholdsMillions)s,\
        %(SolarPenetration)s,\
        %(BatteryPenetration)s,\
        %(BatteryNamePlate)s,\
        %(SolarNamePlate)s,\
        %(RandomMonthFlag)s,\
        %(SpecificMonth)s,\
        %(SolarSpecific)s,\
        %(BatterywSolar)s,\
        %(SpecifiedMonth)s,\
        %(SeasonNum)s,\
        \"%(Season)s\");\
"

parameters = dict(db_params.items() + user_params.items())
# would convert this to use the MySQLdb module,
# but that doesn't allow us to easily source the db dump
mysql_cli(sqlcmds,parameters)

# update the ApplianceInformation, MonthlyTemperature, ClearSkies,
# IrradiationMeansNew, and IrradiationDeviationNew tables with data from the
# user chosen profile.

db = MySQLdb.connect(
        db_params['dbhost'],
        db_params['dbuser'],
        db_params['dbpass'],
        db_params['dbname'])

cursor = db.cursor()

# rebuild the ApplianceInformation table
print "rebuild the ApplianceInformation table with profile %s" % (user_params['AppliancesProfile'])
cursor.execute("DROP TABLE IF EXISTS `ApplianceInformation`;")
cursor.execute("""
    CREATE TABLE `ApplianceInformation` (
      `ProfileName` varchar(510) DEFAULT NULL,
      `ApplianceName` varchar(510) DEFAULT NULL,
      `Saturation` double DEFAULT NULL,
      `StandbyPower` double DEFAULT NULL,
      `_Group` int(11) DEFAULT NULL,
      `Summer` bigint(20) DEFAULT NULL,
      `Winter` bigint(20) DEFAULT NULL
    ) ENGINE=MyISAM DEFAULT CHARSET=latin1;
""")
cursor.execute(
    """select * from ApplianceInformationProfiles
        where ProfileName=%s""", (user_params['AppliancesProfile'],))
results = cursor.fetchall()
cursor.executemany(
    """INSERT INTO `ApplianceInformation` VALUES (%s,%s,%s,%s,%s,%s,%s)""",
    results)


# rebuild the MonthlyTemperature table
print "rebuild the MonthlyTemperature table with profile %s" % (user_params['MonthlyTempProfile'])
cursor.execute("DROP TABLE IF EXISTS `MonthlyTemperature`;")
cursor.execute("""
    CREATE TABLE `MonthlyTemperature` (
      `ProfileName` varchar(510) DEFAULT NULL,
      `MonthofYear` varchar(510) DEFAULT NULL,
      `Max` double DEFAULT NULL,
      `Mean` double DEFAULT NULL,
      `Min` double DEFAULT NULL
    ) ENGINE=MyISAM DEFAULT CHARSET=latin1;
""")
cursor.execute(
    """select * from MonthlyTemperatureProfiles
        where ProfileName=%s""", (user_params['MonthlyTempProfile'],))
results = cursor.fetchall()
cursor.executemany(
    """INSERT INTO `MonthlyTemperature` VALUES (%s,%s,%s,%s,%s)""",
    results)


# rebuild the ClearSkies table
print "rebuild the ClearSkies table with profile %s" % (user_params['CloudCoverProfile'])
cursor.execute("DROP TABLE IF EXISTS `ClearSkies`;")
cursor.execute("""
    CREATE TABLE `ClearSkies` (
      `ProfileName` varchar(510) DEFAULT NULL,
      `Mon` varchar(510) DEFAULT NULL,
      `Proportion` double DEFAULT NULL,
      `Partial` double DEFAULT NULL,
      `ProbTotalCloudy` double DEFAULT NULL
    ) ENGINE=MyISAM DEFAULT CHARSET=latin1;
""")
cursor.execute(
    """select * from ClearSkiesProfiles
        where ProfileName=%s""", (user_params['CloudCoverProfile'],))
results = cursor.fetchall()
cursor.executemany(
    """INSERT INTO `ClearSkies` VALUES (%s,%s,%s,%s,%s)""",
    results)


# rebuild the IrradiationMeansNew table
print "rebuild the IrradiationMeansNew table with profile %s" % (user_params['SolarIrradiationMeansProfile'])
cursor.execute("DROP TABLE IF EXISTS `IrradiationMeansNew`;")
cursor.execute("""
    CREATE TABLE `IrradiationMeansNew` (
      `ProfileName` varchar(510) DEFAULT NULL,
      `January` double DEFAULT NULL,
      `February` double DEFAULT NULL,
      `March` double DEFAULT NULL,
      `April` double DEFAULT NULL,
      `May` double DEFAULT NULL,
      `June` double DEFAULT NULL,
      `July` double DEFAULT NULL,
      `August` double DEFAULT NULL,
      `September` double DEFAULT NULL,
      `October` double DEFAULT NULL,
      `November` double DEFAULT NULL,
      `December` double DEFAULT NULL,
      `Average` double DEFAULT NULL
    ) ENGINE=MyISAM DEFAULT CHARSET=latin1;
""")
cursor.execute(
    """select * from IrradiationMeansNewProfiles
        where ProfileName=%s""", (user_params['SolarIrradiationMeansProfile'],))
results = cursor.fetchall()
cursor.executemany(
    """INSERT INTO `IrradiationMeansNew` VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)""",
    results)


# rebuild the IrradiationDeviationNew table
print "rebuild the IrradiationDeviationNew table with profile %s" % (user_params['SolarIrradiationDevProfile'])
cursor.execute("DROP TABLE IF EXISTS `IrradiationDeviationNew`;")
cursor.execute("""
    CREATE TABLE `IrradiationDeviationNew` (
      `ProfileName` varchar(510) DEFAULT NULL,
      `January` double DEFAULT NULL,
      `February` double DEFAULT NULL,
      `March` double DEFAULT NULL,
      `April` double DEFAULT NULL,
      `May` double DEFAULT NULL,
      `June` double DEFAULT NULL,
      `July` double DEFAULT NULL,
      `August` double DEFAULT NULL,
      `September` double DEFAULT NULL,
      `October` double DEFAULT NULL,
      `November` double DEFAULT NULL,
      `December` double DEFAULT NULL,
      `Average` double DEFAULT NULL
    ) ENGINE=MyISAM DEFAULT CHARSET=latin1;
""")
cursor.execute(
    """select * from IrradiationDeviationNewProfiles
        where ProfileName=%s""", (user_params['SolarIrradiationDevProfile'],))
results = cursor.fetchall()
cursor.executemany(
    """INSERT INTO `IrradiationDeviationNew` VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)""",
    results)



# run the simulation
bin_dir = os.path.dirname(os.path.realpath(__file__))
parent_dir = os.path.join(bin_dir,'SolarPVExported')

cmd = [
    "xvfb-run", "-a", "-s", "-screen 0 800x600x24",
    "/usr/bin/java",
    "-cp", "%(parent_dir)s/*:%(parent_dir)s/lib/*" % {'parent_dir' : parent_dir},
    "-Xmx1000m", "energysystemsaugust2011.Simulation"
]

BUFSIZ=1024
datapoints = []
buf = ''
hours_in_day = 24
days_in_month = 31
lines =  hours_in_day * days_in_month

# kick off the java program
p = subprocess.Popen(cmd,stdout=subprocess.PIPE,shell=False,
        bufsize=BUFSIZ,preexec_fn=os.setsid)

# wait for calculations to complete
while p.poll() is None:
    r,w,x = select.select([p.stdout],[],[])
    for f in r:
        data = os.read(f.fileno(),BUFSIZ)
        buf += data
        elements = buf.split('\n')
        datapoints += [x for x in elements[:-1] if x != '']
        buf = elements[-1]
        sys.stdout.write(data)
        sys.stdout.flush()
    line_count = len(datapoints)
    percent = int(float(line_count)/float(lines)*100.0)
    message = "Simulating" + "." * (line_count % 10)
    Rappture.Utils.progress(percent,message)


Rappture.Utils.progress(100,"Generating graphs")

cursor.execute("select * from TotalNetDemandHour")
results = cursor.fetchall()

# figure out the number of days that was simulated
days = results[0][2:]
days = days[:days.index(None)]
num_days = len(days)

# initialize counters
net_hour_averages = [0] * 24
net_hour_std = [0] * 24
day_sums = [0] * num_days
day_std = [0] * num_days

for row in results:
    hour = int(row[1])
    days_arr = numpy.array(row[2:2+num_days])

    # save the hour averages
    net_hour_averages[hour-1] = numpy.asscalar(days_arr.mean())
    net_hour_std[hour-1] = numpy.asscalar(days_arr.std())

    # save the day sums
    day_sums = map(sum,zip(day_sums,days))


if user_params['SolarPenetration'] != 0.0:
    # plot gross demand (grid + solar)

    cursor.execute("select * from TotalGrossHHDemandHour")
    results = cursor.fetchall()

    # initialize counters
    gross_hour_averages = [0] * 24
    gross_hour_std = [0] * 24

    for row in results:
        hour = int(row[1])
        days_arr = numpy.array(row[2:2+num_days])
        gross_hour_averages[hour-1] = numpy.asscalar(days_arr.mean())
        gross_hour_std[hour-1] = numpy.asscalar(days_arr.std())


cursor.close()
db.close()

# plot the data

driver.put('output.curve(net_hour_demand).about.group','Daily Average Electric Demand By Hour')
driver.put('output.curve(net_hour_demand).about.label','Hourly Net Demand')
driver.put('output.curve(net_hour_demand).about.description',
    'The average electric demand, by hour, only from the power grid')
driver.put('output.curve(net_hour_demand).xaxis.label','Hours of Day')
driver.put('output.curve(net_hour_demand).xaxis.description','')
driver.put('output.curve(net_hour_demand).xaxis.units','hours')
driver.put('output.curve(net_hour_demand).yaxis.label','Energy Demand')
driver.put('output.curve(net_hour_demand).yaxis.description','')
driver.put('output.curve(net_hour_demand).yaxis.units','MWh')
xy = "\n".join(map(lambda x,y: "%i %g" % (x,y),
                   range(len(net_hour_averages)),net_hour_averages))
driver.put('output.curve(net_hour_demand).component.xy',xy)
ye = "\n".join(str(x) for x in net_hour_std)
driver.put('output.curve(net_hour_demand).component.yerrorbars',ye)


if user_params['SolarPenetration'] != 0.0:
    driver.put('output.curve(gross_hour_demand).about.group','Daily Average Electric Demand By Hour')
    driver.put('output.curve(gross_hour_demand).about.label','Hourly Gross Demand')
    driver.put('output.curve(gross_hour_demand).about.description',
        'The average electric demand, by hour, only from the power grid')
    driver.put('output.curve(gross_hour_demand).xaxis.label','Hours of Day')
    driver.put('output.curve(gross_hour_demand).xaxis.description','')
    driver.put('output.curve(gross_hour_demand).xaxis.units','hours')
    driver.put('output.curve(gross_hour_demand).yaxis.label','Energy Demand')
    driver.put('output.curve(gross_hour_demand).yaxis.description','')
    driver.put('output.curve(gross_hour_demand).yaxis.units','MWh')
    xy = "\n".join(map(lambda x,y: "%i %g" % (x,y),
                       range(len(gross_hour_averages)),gross_hour_averages))
    driver.put('output.curve(gross_hour_demand).component.xy',xy)
    ye = "\n".join(str(x) for x in gross_hour_std)
    driver.put('output.curve(gross_hour_demand).component.yerrorbars',ye)


driver.put('output.curve(total_daily_demand).about.label','Total Daily Electric Demand In A Month')
driver.put('output.curve(total_daily_demand).about.description','Total electric demand each day')
driver.put('output.curve(total_daily_demand).xaxis.label','Days of Month')
driver.put('output.curve(total_daily_demand).xaxis.description','')
driver.put('output.curve(total_daily_demand).xaxis.units','day')
driver.put('output.curve(total_daily_demand).yaxis.label','Total Electric Demand')
driver.put('output.curve(total_daily_demand).yaxis.description','')
driver.put('output.curve(total_daily_demand).yaxis.units','MWh')
xy = "\n".join(map(lambda x,y: "%i %g" % (x,y), range(len(day_sums)),day_sums))
driver.put('output.curve(total_daily_demand).component.xy',xy)



Rappture.result(driver)
sys.exit()
